import time

from OpenSSL.crypto import PKey


class FakeMessage:
    def __init__(self, mid):
        self.mid = mid
        self.date = int(time.time() - 1000)


class FakeDialog:
    def __init__(self, user_peers=None, group_peers=None):
        if user_peers:
            self.user_peers = user_peers
        else:
            self.user_peers = [FakeOutpeer()]
        if group_peers:
            self.group_peers = group_peers
        else:
            self.group_peers = [FakeOutpeer()]


class FakeOutpeer:
    def __init__(self):
        self.uid = 0
        self.access_hash = 0


class FakeUsers:
    def __init__(self, nick):
        self.users = [FakeUser(nick)]


class FakeUser:
    def __init__(self, nick):
        self.data = FakeUserData(nick)
        self.id = 0
        self.access_hash = 0


class FakeUserData:
    def __init__(self, nick):
        self.nick = FakeUserDataValue(nick)


class FakeUserDataValue:
    def __init__(self, nick):
        self.value = nick


class FakeFullProfile:
    def __init__(self, full_users=None):
        if full_users:
            self.full_users = full_users


class FakeFullUser:
    def __init__(self, custom_profile=None):
        if custom_profile:
            self.custom_profile = custom_profile


class FakeSearch:
    def __init__(self):
        self.groups = ["group"]
        self.peer = "peer"


class FakeState:
    def __init__(self):
        self.seq = 1


class FakeUrl:
    def __init__(self):
        self.url = "url"


class FakePut:
    def __init__(self, status_code=None, upload_key=None, uploaded_file_location=None):
        self.status_code = status_code
        self.upload_key = upload_key
        self.uploaded_file_location = uploaded_file_location


class FakePKey:
    def get_privatekey(self):
        return PKey()

    def get_certificate(self):
        return



import unittest

from dialog_api import messaging_pb2, peers_pb2
from dialog_api.peers_pb2 import PEERTYPE_PRIVATE
from mock import patch

from dialog_bot_sdk import interactive_media
from tests.bot import bot


class TestInteractive(unittest.TestCase):
    def test_confirm(self):
        confirm = interactive_media.InteractiveMediaConfirm("text", "title", "ok", "dismiss")
        self.assertTrue(isinstance(confirm.render(), messaging_pb2.InteractiveMediaConfirm))
        confirm = interactive_media.InteractiveMediaConfirm("text", "title", "ok")
        self.assertTrue(isinstance(confirm.render(), messaging_pb2.InteractiveMediaConfirm))
        confirm = interactive_media.InteractiveMediaConfirm("text", "title")
        self.assertTrue(isinstance(confirm.render(), messaging_pb2.InteractiveMediaConfirm))
        confirm = interactive_media.InteractiveMediaConfirm("text")
        self.assertTrue(isinstance(confirm.render(), messaging_pb2.InteractiveMediaConfirm))
        confirm = interactive_media.InteractiveMediaConfirm()
        self.assertTrue(isinstance(confirm.render(), messaging_pb2.InteractiveMediaConfirm))

    @patch('dialog_bot_sdk.messaging.Messaging._send_message')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    def test_button_and_media(self, get, send):
        get.return_value = peers_pb2.OutPeer(type=PEERTYPE_PRIVATE, id=0, access_hash=0)
        confirm = interactive_media.InteractiveMediaConfirm()
        button = interactive_media.InteractiveMediaButton("value", "label")
        interactive = interactive_media.InteractiveMedia(1, button, 'default', confirm)
        group = interactive_media.InteractiveMediaGroup([interactive])
        bot.messaging.send_message("peer", "text", [group])
        media = send.call_args.args[0].message.textMessage.media[0]
        self.assertTrue(isinstance(media, messaging_pb2.MessageMedia))
        actions_group = media.actions[0]
        self.assertTrue(isinstance(actions_group, messaging_pb2.InteractiveMediaGroup))
        actions = actions_group.actions[0]
        self.assertTrue(isinstance(actions, messaging_pb2.InteractiveMedia))
        self.assertTrue(isinstance(actions.widget.interactiveMediaButton, messaging_pb2.InteractiveMediaButton))
        self.assertEqual(actions.id, '1')
        self.assertEqual(actions.style, 1)
        self.assertTrue(isinstance(actions.confirm, messaging_pb2.InteractiveMediaConfirm))
        self.assertEqual(actions.widget.interactiveMediaButton.value, 'value')
        self.assertEqual(actions.widget.interactiveMediaButton.label.value, 'label')

    @patch('dialog_bot_sdk.messaging.Messaging._send_message')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    def test_select_and_media(self, get, send):
        get.return_value = peers_pb2.OutPeer(type=PEERTYPE_PRIVATE, id=0, access_hash=0)
        with self.assertRaises(AttributeError):
            interactive_media.InteractiveMediaSelect(None)
        select = interactive_media.InteractiveMediaSelect({"yes": "true", "no": "false"}, "yes", "true")
        interactive = interactive_media.InteractiveMedia(1, select)
        group = interactive_media.InteractiveMediaGroup([interactive])
        bot.messaging.send_message("peer", "text", [group])
        i_select = send.call_args.args[0].message.textMessage.media[0].actions[0].actions[0].\
            widget.interactiveMediaSelect
        self.assertTrue(isinstance(i_select, messaging_pb2.InteractiveMediaSelect))
        self.assertTrue(isinstance(i_select.options[0], messaging_pb2.InteractiveMediaSelectOption))
        self.assertEqual(i_select.label.value, 'yes')
        self.assertEqual(i_select.default_value.value, 'true')

    @patch('dialog_bot_sdk.messaging.Messaging._send_message')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    def test_group(self, get, send):
        get.return_value = peers_pb2.OutPeer(type=PEERTYPE_PRIVATE, id=0, access_hash=0)
        with self.assertRaises(AttributeError):
            interactive_media.InteractiveMediaGroup("not_list")
        button = interactive_media.InteractiveMediaButton("value", "label")
        interactive = interactive_media.InteractiveMedia(1, button)
        group = interactive_media.InteractiveMediaGroup([interactive], "title", "description",
                                                        {"lang": {"id": "value"}})
        bot.messaging.send_message("peer", "text", [group])
        group_args = send.call_args.args[0].message.textMessage.media[0].actions[0]
        self.assertEqual(group_args.description.value, 'description')
        self.assertEqual(group_args.title.value, 'title')
        self.assertTrue(isinstance(group_args.translations[0], messaging_pb2.InteractiveMediaTranslationGroup))
        self.assertEqual(group_args.translations[0].language, "lang")
        self.assertEqual(group_args.translations[0].messages[0].id, 'id')
        self.assertEqual(group_args.translations[0].messages[0].value, 'value')


if __name__ == '__main__':
    unittest.main()



import cv2
import mimetypes


def get_video_w_h(file):
    """Returns video's width and height

    :param file: path to video
    :return: width, height
    """
    vid = cv2.VideoCapture(file)
    height = int(vid.get(cv2.CAP_PROP_FRAME_HEIGHT))
    width = int(vid.get(cv2.CAP_PROP_FRAME_WIDTH))
    return width, height


def is_video(file):
    mime = mimetypes.guess_type(file)[0]
    return mime in ['video/x-msvideo', 'video/mpeg', 'video/quicktime', 'video/x-sgi-movie', 'video/mp4',
                    'video/quicktime', 'video/webm']



import random

from google.protobuf import wrappers_pb2

from .service import ManagedService
from dialog_api import search_pb2, groups_pb2, peers_pb2, sequence_and_updates_pb2, media_and_files_pb2


class Groups(ManagedService):
    """Class for handling groups

    """
    permissions_map = {
        'edit_shortname': groups_pb2.GROUPADMINPERMISSION_EDITSHORTNAME,
        'invite': groups_pb2.GROUPADMINPERMISSION_INVITE,
        'kick': groups_pb2.GROUPADMINPERMISSION_KICK,
        'update_info': groups_pb2.GROUPADMINPERMISSION_UPDATEINFO,
        'set_permissions': groups_pb2.GROUPADMINPERMISSION_SETPERMISSIONS,
        'edit_message': groups_pb2.GROUPADMINPERMISSION_EDITMESSAGE,
        'delete_message': groups_pb2.GROUPADMINPERMISSION_DELETEMESSAGE,
        'get_integration_token': groups_pb2.GROUPADMINPERMISSION_GETINTEGRATIONTOKEN,
        'send_message': groups_pb2.GROUPADMINPERMISSION_SENDMESSAGE,
        'pin_message': groups_pb2.GROUPADMINPERMISSION_PINMESSAGE,
        'view_members': groups_pb2.GROUPADMINPERMISSION_VIEWMEMBERS,
    }

    def create_public_group(self, title, shortname):
        """Create public group

        :param title: title of group
        :param shortname: group name
        :return: group
        """

        request = groups_pb2.RequestCreateGroup(
            title=title,
            username=wrappers_pb2.StringValue(value=shortname),
            group_type=groups_pb2.GROUPTYPE_GROUP
        )
        return self._create_group(request)

    def create_private_group(self, title):
        """Create private group

        :param title: title of group
        :return: group
        """

        request = groups_pb2.RequestCreateGroup(
            title=title,
            group_type=groups_pb2.GROUPTYPE_GROUP
        )
        return self._create_group(request)

    def create_public_channel(self, title, shortname):
        """Create public channel

        :param title: title of group
        :param shortname: group name
        :return: group
        """

        request = groups_pb2.RequestCreateGroup(
            title=title,
            username=wrappers_pb2.StringValue(value=shortname),
            group_type=groups_pb2.GROUPTYPE_CHANNEL
        )
        return self._create_group(request)

    def create_private_channel(self, title):
        """Create private channel

        :param title: title of group
        :return: group
        """

        request = groups_pb2.RequestCreateGroup(
            title=title,
            group_type=groups_pb2.GROUPTYPE_CHANNEL
        )
        return self._create_group(request)

    def find_group_by_shortname(self, query):
        """Find a groups by shortname

        :param query: shortname of group
        :return: Group
        """
        request = search_pb2.RequestPeerSearch(
            query=[
                search_pb2.SearchCondition(
                    searchPeerTypeCondition=search_pb2.SearchPeerTypeCondition(
                        peer_type=search_pb2.SEARCHPEERTYPE_GROUPS
                    )
                ),
                search_pb2.SearchCondition(
                    searchPieceText=search_pb2.SearchPieceText(query=query)
                )
            ]
        )
        response = self._peer_search(request).search_results
        for result in response:
            if result.peer.type == peers_pb2.PEERTYPE_GROUP:
                if hasattr(result, 'shortname'):
                    if result.shortname.value == query:
                        return result

    def load_members(self, group_peer, limit=0):
        """User's list from group

        :param group_peer: group's peer
        :param limit: count members
        :return: list of User's
        """
        group_out_peer = self.get_group_outpeer(group_peer.id)
        request = groups_pb2.RequestLoadMembers(
            group=group_out_peer,
            limit=limit
        )
        members = self._load_members(request).members
        result = self.internal.updates.GetReferencedEntitites(
            sequence_and_updates_pb2.RequestGetReferencedEntitites(
                group_members=[
                    sequence_and_updates_pb2.GroupMembersSubset(
                        group_peer=group_out_peer,
                        member_ids=[member.uid for member in members]
                    )
                ]
            )
        )

        return result

    def kick_user(self, group_peer, user):
        """return response of KickUser

        :param group_peer: group's peer
        :param user: user's OutPeer
        :return: response
        """
        group_out_peer, user_out_peer = self.get_group_outpeer(group_peer.id), self.get_user_outpeer(user)
        return self.internal.groups.KickUser(
            groups_pb2.RequestKickUser(
                group_peer=group_out_peer,
                user=user_out_peer,
                rid=random.randint(0, 100000000),
            )
        )

    def invite_user(self, group_peer, user):
        """return response of InviteUser

        :param group_peer: group's peer
        :param user: user's OutPeer
        :return: response
        """
        group_out_peer = self.get_group_outpeer(group_peer.id)
        user_out_peer = self.get_user_outpeer(user)
        return self.internal.groups.InviteUser(
            groups_pb2.RequestInviteUser(
                group_peer=group_out_peer,
                user=user_out_peer,
                rid=random.randint(0, 100000000),
            )
        )

    def kick_users(self, group_peer, users):
        """return response list of KickUser

        :param group_peer: group's peer
        :param users: user's OutPeers
        :return: response list
        """
        result = []
        for user in users:
            result.append(self.kick_user(group_peer, user))
        return result

    def invite_users(self, group_peer, users):
        """return response list of InviteUser

        :param group_peer: group's peer
        :param users: user's OutPeers
        :return: response list
        """
        result = []
        for user in users:
            result.append(self.invite_user(group_peer, user))
        return result

    def set_default_group_permissions(self, group_peer, add_permissions=None, del_permissions=None):
        """add/del default group permissions

        :param group_peer: group's peer
        :param add_permissions: list of permissions (on permissions_map) to add
        :param del_permissions: list of permissions (on permissions_map) to delete
        :return: response list
        """
        if del_permissions is None:
            del_permissions = []
        if add_permissions is None:
            add_permissions = []
        result = []
        group_out_peer = self.get_group_outpeer(group_peer.id)
        for permission in add_permissions:
            request = groups_pb2.RequestEditGroupBasePermissions(
                    group_peer=group_out_peer,
                    random_id=random.randint(0, 100000000),
                    granted_permissions=[self.permissions_map[permission]]
                )
            result.append(self._set_default_group_permissions(request))
        for permission in del_permissions:
            request = groups_pb2.RequestEditGroupBasePermissions(
                    group_peer=group_out_peer,
                    random_id=random.randint(0, 100000000),
                    revoked_permissions=[self.permissions_map[permission]]
                )
            result.append(self._set_default_group_permissions(request))
        return result

    def set_member_permissions(self, group_peer, user, add_permissions=None, del_permissions=None):
        """add/del group's member permissions

        :param group_peer: group's peer
        :param user: OutPeer of user
        :param add_permissions: list of permissions (on permissions_map) to add
        :param del_permissions: list of permissions (on permissions_map) to delete
        :return: response list
        """
        if del_permissions is None:
            del_permissions = []
        if add_permissions is None:
            add_permissions = []
        group_out_peer, user_out_peer = self.get_group_outpeer(group_peer.id), self.get_user_outpeer(user)
        result = []
        for permission in add_permissions:
            request = groups_pb2.RequestEditMemberPermissions(
                    group_peer=group_out_peer,
                    user_peer=user_out_peer,
                    granted_permissions=[self.permissions_map[permission]]
                )
            result.append(self._set_member_permissions(request))
        for permission in del_permissions:
            request = groups_pb2.RequestEditMemberPermissions(
                    group_peer=group_out_peer,
                    user_peer=user_out_peer,
                    revoked_permissions=[self.permissions_map[permission]]
                )
            result.append(self._set_member_permissions(request))
        return result

    def edit_avatar(self, group_id, file):
        outpeer = self.get_group_outpeer(group_id)
        location = self.internal.uploading.upload_file(file)
        request = groups_pb2.RequestEditGroupAvatar(
            group_peer=outpeer,
            rid=random.randint(0, 100000000),
            file_location=location
        )
        return self._edit_group_avatar(request)

    def get_group_outpeer(self, group_id):
        """return GroupOutPeer object

        :param group_id: group's id
        :return: GroupOutPeer
        """
        gop = self.manager.get_outpeer(peers_pb2.Peer(id=group_id, type=peers_pb2.PEERTYPE_GROUP))
        return peers_pb2.GroupOutPeer(group_id=gop.id, access_hash=gop.access_hash)

    def get_outpeer(self, group_id):
        """return OutPeer object

        :param group_id: group's id
        :return: OutPeer
        """
        request = peers_pb2.Peer(id=group_id, type=peers_pb2.PEERTYPE_GROUP)
        return self.manager.get_outpeer(request)

    @staticmethod
    def get_user_outpeer(user):
        """return UserOutPeer object

        :param user: OutPeer
        :return: UserOutPeer
        """
        return peers_pb2.UserOutPeer(uid=user.id, access_hash=user.access_hash)

    def _create_group(self, request):
        group = self.internal.groups.CreateGroup(request).group
        self.manager.adopt_peer(peers_pb2.GroupOutPeer(group_id=group.id, access_hash=group.access_hash))
        return group

    def _peer_search(self, request):
        return self.internal.search.PeerSearch(request)

    def _load_members(self, request):
        return self.internal.groups.LoadMembers(request)

    def _set_default_group_permissions(self, request):
        return self.internal.groups.EditGroupBasePermissions(request)

    def _set_member_permissions(self, request):
        return self.internal.groups.EditMemberPermissions(request)

    def _edit_group_avatar(self, request):
        return self.internal.groups.EditGroupAvatar(request)



from PIL import Image
import mimetypes
import io


def get_image_w_h(file):
    """Returns image's width and height

    :param file: path to image
    :return: (width, height) tuple
    """

    im = Image.open(file)
    return im.size


def get_image_thumb_bytes(file):
    """Returns image's thumbnail in form of byte array

    :param file: path to image
    :return: size and byte array
    """

    size = 90, 90

    im = Image.open(file)
    im = im.convert('RGB')
    im.thumbnail(size)
    stream = io.BytesIO()
    im.save(stream, "JPEG")

    return size[0], size[1], stream.getvalue()


def is_image(file):
    mime = mimetypes.guess_type(file)[0]
    return mime in ['image/jpeg', 'image/png', 'image/gif', 'image/bmp']



from .service import ManagedService
from dialog_api import sequence_and_updates_pb2


class Updates(ManagedService):
    """Class for handling grpc's server updates.

    """
    def get_difference(self, seq):
        """Raw implementation of API schema GetDifference method, which returns updates between attribute 'seq' and
        current seq at server side.

        :param seq: seq value
        :return: GetDifferenceResponse object
        """
        request = sequence_and_updates_pb2.RequestGetDifference(seq=seq)

        return self._get_difference(request)

    def get_state(self):
        """Current application seq number

        :return: seq
        """
        return self._get_state(sequence_and_updates_pb2.RequestGetState()).seq

    def _get_difference(self, request):
        return self.internal.updates.GetDifference(request)

    def _get_state(self, request):
        return self.internal.updates.GetState(request)




import unittest

from dialog_api import media_and_files_pb2, messaging_pb2
from google.protobuf import wrappers_pb2
from mock import patch
from dialog_bot_sdk.utils import get_media

from tests.bot import bot


class TestGetMedia(unittest.TestCase):
    def test_get_str_val(self):
        self.assertTrue(isinstance(get_media.get_str_val("string"), wrappers_pb2.StringValue))

    def test_get_webpage(self):
        self.assertTrue(isinstance(get_media.get_webpage("https"), messaging_pb2.MessageMedia))
        self.assertTrue(isinstance(get_media.get_webpage("https", "title"), messaging_pb2.MessageMedia))
        self.assertTrue(isinstance(get_media.get_webpage("https", "title", "description"), messaging_pb2.MessageMedia))
        image_location = media_and_files_pb2.ImageLocation(file_location=
                                                           media_and_files_pb2.FileLocation(file_id=0, access_hash=0),
                                                           width=100, height=100)
        self.assertTrue(isinstance(get_media.get_webpage("https", "title", "description", image_location),
                                   messaging_pb2.MessageMedia))

    @patch('dialog_bot_sdk.uploading.Uploading.upload_file')
    def test_get_image_location(self, upload):
        with self.assertRaises(IOError):
            get_media.get_image_location(bot, "./files/test.mp3")
        upload.return_value = media_and_files_pb2.FileLocation(file_id=0, access_hash=0)
        self.assertTrue(isinstance(get_media.get_image_location(bot, "./files/test.png"),
                                   media_and_files_pb2.ImageLocation))
        self.assertEqual(upload.call_args.args[0], "./files/test.png")
        self.assertTrue(isinstance(get_media.get_image_location(bot, "./files/test.png", 100),
                                   media_and_files_pb2.ImageLocation))
        self.assertTrue(isinstance(get_media.get_image_location(bot, "./files/test.png", 100, 100),
                                   media_and_files_pb2.ImageLocation))

    @patch('dialog_bot_sdk.utils.get_media.get_image_location')
    def test_get_image(self, get):
        get.return_value = media_and_files_pb2.ImageLocation(file_location=
                                                             media_and_files_pb2.FileLocation(file_id=0, access_hash=0),
                                                             width=100, height=100)
        self.assertTrue(isinstance(get_media.get_image(bot, "./files/test.png"),
                                   messaging_pb2.MessageMedia))
        self.assertTrue(isinstance(get_media.get_image(bot, "./files/test.png", 100),
                                   messaging_pb2.MessageMedia))
        self.assertTrue(isinstance(get_media.get_image(bot, "./files/test.png", 100, 100),
                                   messaging_pb2.MessageMedia))
        self.assertEqual(get.call_args.args, (bot, "./files/test.png", 100, 100))

    @patch('dialog_bot_sdk.uploading.Uploading.upload_file')
    def test_get_audio(self, upload):
        with self.assertRaises(IOError):
            get_media.get_audio(bot, "./files/test.png")
            upload.return_value = media_and_files_pb2.FileLocation(file_id=0, access_hash=0)
            self.assertTrue(isinstance(get_media.get_audio(bot, "./files/test.mp3"),
                                       messaging_pb2.MessageMedia))
            self.assertTrue(isinstance(get_media.get_audio(bot, "./files/test.mp3", 2),
                                       messaging_pb2.MessageMedia))
            self.assertEqual(upload.call_args.args, (bot, "./files/test.mp3", 2))


if __name__ == '__main__':
    unittest.main()



from dialog_api import messaging_pb2


class InteractiveMediaButton(object):
    """Button control class.

    """
    def __init__(self, value, label=None):
        self.value = value
        self.label = label

    def render(self, target):
        """Render method for button

        :param target: target button
        """
        if self.label is not None:
            target.label.value = self.label
        target.value = self.value


class InteractiveMediaSelect(object):
    """Select control class.

    """
    def __init__(self, options, label=None, default_value=None):
        if options is None:
            raise AttributeError('Attribute \'options\' can\'t be None.')

        self.options = options
        self.label = label
        self.default_value = default_value

    def render(self, target):
        """Render method for select

        :param target: target select
        """
        if self.label is not None:
            target.label.value = self.label
        if self.default_value is not None:
            target.default_value.value = self.default_value
        for value, label in self.options.items():
            opt = target.options.add()
            opt.value = value
            opt.label = label


class InteractiveMediaConfirm(object):
    """Confirm control class.

    """
    def __init__(self, text=None, title=None, ok=None, dismiss=None):
        self.text = text
        self.title = title
        self.ok = ok
        self.dismiss = dismiss

    def render(self):
        """Render method for confirm

        :return: confirm
        """
        confirm = messaging_pb2.InteractiveMediaConfirm()
        if self.text is not None:
            confirm.text.value = self.text
        if self.title is not None:
            confirm.title.value = self.title
        if self.ok is not None:
            confirm.ok.value = self.ok
        if self.dismiss is not None:
            confirm.dismiss.value = self.dismiss
        return confirm


class InteractiveMedia(object):
    """Wrapper class for interactive object styling.

    """
    style_map = {
        'default': messaging_pb2.INTERACTIVEMEDIASTYLE_DEFAULT,
        'primary': messaging_pb2.INTERACTIVEMEDIASTYLE_PRIMARY,
        'danger': messaging_pb2.INTERACTIVEMEDIASTYLE_DANGER
    }
    # style one of ['default', 'primary', 'danger', None]
    # widget = InteractiveMediaButton | InteractiveMediaSelect

    def __init__(self, media_id, widget, style=None, confirm=None):
        self.media_id = media_id
        self.widget = widget
        self.style = style
        self.confirm = confirm

    def render(self, target):
        """Render method for wrapped interactive object.

        :param target: target interactive object
        :return: wrapped interactive object
        """
        target.id = str(self.media_id)
        target.style = self.style_map.get(self.style, messaging_pb2.INTERACTIVEMEDIASTYLE_UNKNOWN)
        if self.widget is not None:
            if isinstance(self.widget, InteractiveMediaButton):
                self.widget.render(target.widget.interactiveMediaButton)
            elif isinstance(self.widget, InteractiveMediaSelect):
                self.widget.render(target.widget.interactiveMediaSelect)
        if self.confirm is not None:
            result_confirm = self.confirm.render()
            target.confirm.text.value = result_confirm.text.value
            target.confirm.title.value = result_confirm.title.value
            target.confirm.ok.value = result_confirm.ok.value
            target.confirm.dismiss.value = result_confirm.dismiss.value

        return target


class InteractiveMediaGroup(object):
    """Wrapper class for interactive object grouping.

    """
    def __init__(self, actions, title=None, description=None, translations=None):
        if not isinstance(actions, list):
            raise AttributeError('Actions must be an iterable.')

        self.actions = actions
        self.title = title
        self.description = description
        self.translations = translations

    def render(self, target):
        """Render method for group of interactive objects.

        :param target: group of interactive objects
        """
        media = target.actions.add()
        for action in self.actions:
            act = media.actions.add()
            action.render(act)
        if self.title is not None:
            media.title.value = self.title
        if self.description is not None:
            media.description.value = self.description
        if self.translations:
            for lang, trans in self.translations.items():
                group = messaging_pb2.InteractiveMediaTranslationGroup(language=lang)
                for idx, value in trans.items():
                    group.messages.append(messaging_pb2.InteractiveMediaTranslation(id=idx, value=value))
                media.translations.append(group)



import google.protobuf.wrappers_pb2 as wrappers_pb2
from dialog_api import messaging_pb2, media_and_files_pb2
import mimetypes

from dialog_bot_sdk.utils.get_audio_metadata import get_audio_duration, is_audio
from dialog_bot_sdk.utils.get_image_metadata import is_image


def get_str_val(s):
    """Return obj google.protobuf.StringValue

    :param s: string
    :return: StringValue
    """
    return wrappers_pb2.StringValue(value=s)


def get_webpage(url, title=None, description=None, image_location=None):
    """Return MessageMedia with WebpageMedia for messaging.send_media

    :param url: url (str)
    :param title: title (str)
    :param description: description (str)
    :param image_location: image (ImageLocation)
    :return: MessageMedia obj
    """
    return messaging_pb2.MessageMedia(
        webpage=messaging_pb2.WebpageMedia(
            url=get_str_val(url),
            title=get_str_val(title),
            description=get_str_val(description),
            image=image_location
        )
    )


def get_image_location(bot, file, width=100, height=100):
    """Return obj ImageLocation

    :param bot: DialogBot
    :param file: image's file
    :param width: image's width
    :param height: image's height
    :return: ImageLocation
    """
    if not is_image(file):
        raise IOError('File is not an image.')
    location = bot.internal.uploading.upload_file(file)
    return media_and_files_pb2.ImageLocation(file_location=location, width=width, height=height)


def get_image(bot, file, width=100, height=100):
    """Return MessageMedia with ImageMedia for messaging.send_media

    :param bot: DialogBot
    :param file: image's file
    :param width: subj
    :param height: subj
    :return: MessageMedia obj
    """
    image_location = get_image_location(bot, file, width, height)
    return messaging_pb2.MessageMedia(image=messaging_pb2.ImageMedia(image=image_location))


def get_audio(bot, file, duration=0):
    """Return MessageMedia with AudioMedia for messaging.send_media

    :param bot: DialogBot
    :param file: audio's file
    :param duration: duration audio
    :return: MessageMedia obj
    """
    if not is_audio(file):
        raise IOError('File is not an audio.')
    mime_type = mimetypes.guess_type(file)[0]
    if not duration:
        duration = get_audio_duration(file)
    file_location = bot.internal.uploading.upload_file(file)
    return messaging_pb2.MessageMedia(
        audio=messaging_pb2.AudioMedia(
            audio=media_and_files_pb2.AudioLocation(
                file_location=file_location, mime_type=mime_type, duration=duration))
    )



import unittest

from dialog_api import contacts_pb2, peers_pb2, messaging_pb2
from dialog_api.peers_pb2 import PEERTYPE_PRIVATE, PEERTYPE_GROUP
from mock import patch

from tests.bot import bot
from tests.fake_classes import FakeDialog, FakeOutpeer
from dialog_bot_sdk import entity_manager


class TestManager(unittest.TestCase):

    user_out_peer = peers_pb2.UserOutPeer(uid=0, access_hash=0)
    group_out_peer = peers_pb2.GroupOutPeer(group_id=1, access_hash=-1)
    out_peer = peers_pb2.OutPeer(id=0, access_hash=0)
    peer = peers_pb2.Peer(type=PEERTYPE_PRIVATE, id=0)
    group_peer = peers_pb2.Peer(type=PEERTYPE_GROUP, id=1)

    @patch('dialog_bot_sdk.entity_manager.EntityManager.adopt_peer')
    @patch('dialog_bot_sdk.entity_manager.EntityManager._get_contacts')
    def test_bootstrap(self, contacts, peer):
        contacts.return_value = FakeDialog()
        entity_manager.EntityManager(bot.internal)
        self.assertTrue(isinstance(contacts.call_args.args[0], contacts_pb2.RequestGetContacts))
        self.assertTrue(isinstance(peer.call_args.args[0], FakeOutpeer))

    def test_adopt_peer(self):
        manager = entity_manager.EntityManager(bot.internal)
        manager.adopt_peer(self.user_out_peer)
        manager.adopt_peer(self.group_out_peer)
        self.assertTrue(isinstance(manager.peers_to_outpeers[(1, 0)], peers_pb2.OutPeer))
        self.assertEqual(manager.peers_to_outpeers[(1, 0)].type, 1)
        self.assertTrue(isinstance(manager.peers_to_outpeers[(2, 1)], peers_pb2.OutPeer))
        self.assertEqual(manager.peers_to_outpeers[(2, 1)].type, 2)
        with self.assertRaises(RuntimeError):
            manager.adopt_peer("no_peer")

    @patch('dialog_bot_sdk.entity_manager.EntityManager._load_dialogs')
    def test_get_outpeer(self, load):
        manager = entity_manager.EntityManager(bot.internal)

        self.assertEqual(self.out_peer, manager.get_outpeer(self.out_peer))

        load.return_value = FakeDialog(user_peers=[self.user_out_peer], group_peers=[self.group_out_peer])

        self.assertTrue(isinstance(manager.get_outpeer(self.peer), peers_pb2.OutPeer))
        self.assertTrue(isinstance(manager.get_outpeer(self.group_peer), peers_pb2.OutPeer))
        self.assertTrue(isinstance(load.call_args.args[0], messaging_pb2.RequestLoadDialogs))




if __name__ == '__main__':
    unittest.main()



from .service import ManagedService
from dialog_api import peers_pb2, users_pb2, messaging_pb2, search_pb2, sequence_and_updates_pb2


class Users(ManagedService):
    """Class for handling users
    """
    def find_user_outpeer_by_nick(self, nick):
        """Return OutPeer by shortname
        :param nick: nick (str)
        :return: OutPeer object
        """
        result = self.internal.search.ResolvePeer(search_pb2.RequestResolvePeer(
                shortname=nick
            )
        )
        if hasattr(result, "peer"):
            return result.peer

    def get_user_outpeer_by_id(self, uid):
        req = messaging_pb2.RequestLoadDialogs(
            min_date=0,
            limit=1,
            peers_to_load=[peers_pb2.Peer(type=peers_pb2.PEERTYPE_PRIVATE, id=uid)]
        )
        result = self.internal.messaging.LoadDialogs(req)

        for outpeer in result.user_peers:
            if outpeer.uid == uid:
                return peers_pb2.OutPeer(
                    type=peers_pb2.PEERTYPE_PRIVATE,
                    id=outpeer.uid,
                    access_hash=outpeer.access_hash
                )

    @staticmethod
    def get_user_peer_by_id(uid):
        return peers_pb2.Peer(type=peers_pb2.PEERTYPE_PRIVATE, id=uid)

    def get_user_by_nick(self, nick):
        """Returns User object by nickname
        :param nick: user's nickname
        :return: User object
        """
        outpeer = self.find_user_outpeer_by_nick(nick)
        if not (outpeer.id and outpeer.access_hash):
            return

        result = self.internal.updates.GetReferencedEntitites(
            sequence_and_updates_pb2.RequestGetReferencedEntitites(
                users=[
                    peers_pb2.UserOutPeer(uid=outpeer.id, access_hash=outpeer.access_hash)
                ]
            )
        )
        for user in result.users:
            if hasattr(user.data, "nick") and user.data.nick.value == nick:
                return user

    def get_user_full_profile_by_nick(self, nick):
        """Returns FullUser object by nickname
        :param nick: user's nickname
        :return: FullUser object
        """
        user = self.find_user_outpeer_by_nick(nick)
        full_profile = self.internal.users.LoadFullUsers(
            users_pb2.RequestLoadFullUsers(
                user_peers=[
                    peers_pb2.UserOutPeer(
                        uid=user.id,
                        access_hash=user.access_hash
                    )
                ]
            )
        )

        if full_profile:
            if hasattr(full_profile, 'full_users'):
                if len(full_profile.full_users) > 0:
                    return full_profile.full_users[0]

    def get_user_custom_profile_by_nick(self, nick):
        """Returns custom_profile field of FullUser object by nickname
        :param nick: user's nickname
        :return: user's custom profile string
        """
        full_profile = self.get_user_full_profile_by_nick(nick)

        if hasattr(full_profile, 'custom_profile'):
            return str(full_profile.custom_profile)

    def get_user_custom_profile_by_peer(self, peer):
        """Returns custom_profile field of FullUser object by Peer object
        :param peer: user's Peer object
        :return: user's custom profile string
        """
        outpeer = self.manager.get_outpeer(peer)

        full_profile = self.internal.users.LoadFullUsers(
            users_pb2.RequestLoadFullUsers(
                user_peers=[
                    peers_pb2.UserOutPeer(
                        uid=outpeer.id,
                        access_hash=outpeer.access_hash
                    )
                ]
            )
        )

        if full_profile:
            if hasattr(full_profile, 'full_users'):
                if len(full_profile.full_users) > 0:
                    if hasattr(full_profile.full_users[0], 'custom_profile'):
                        return str(full_profile.full_users[0].custom_profile)

    def search_users_by_nick_substring(self, query):
        """Returns list of User objects by substring of nickname (not complete coincidence!)
        :param query: user's nickname
        :return: list User objects
        """
        user_peers = self.internal.search.PeerSearch(
            search_pb2.RequestPeerSearch(
                query=[
                    search_pb2.SearchCondition(
                        searchPeerTypeCondition=search_pb2.SearchPeerTypeCondition(
                            peer_type=search_pb2.SEARCHPEERTYPE_CONTACTS)
                    ),
                    search_pb2.SearchCondition(
                        searchPieceText=search_pb2.SearchPieceText(query=query)
                    )
                ]
            )
        ).user_peers
        users_list = self.internal.updates.GetReferencedEntitites(
            sequence_and_updates_pb2.RequestGetReferencedEntitites(
                users=list(user_peers)
            )
        )
        result = []

        for user in users_list.users:
            if hasattr(user.data, "nick") and query in user.data.nick.value:
                result.append(user)
        return result

    def _get_referenced_entitites(self, request):
        return self.internal.updates.GetReferencedEntitites(request)


import time

from google.protobuf import empty_pb2
import threading
import random
import grpc

from .service import ManagedService
from dialog_api import messaging_pb2, sequence_and_updates_pb2
from .content import content
from .utils.get_image_metadata import is_image
from .utils.get_video_metadata import is_video
import google.protobuf.wrappers_pb2 as wrappers_pb2


class Messaging(ManagedService):
    """Main messaging class.
    """
    def send_message(self, peer, text, interactive_media_groups=None, uid=None):
        """Send text message to current peer. Message can contain interactive media groups (buttons, selects etc.).
        :param peer: receiver's peer
        :param text: message text (not null)
        :param interactive_media_groups: groups of interactive media components (buttons etc.)
        :param uid: send message only for user by id
        :return: value of SendMessage response object
        """

        if text == '' or text is None:
            raise AttributeError('Text message must contain some text.')

        if not peer:
            print('Peer can\'t be None!')
            return None

        outpeer = self.manager.get_outpeer(peer)
        msg = messaging_pb2.MessageContent()
        msg.textMessage.text = text
        if interactive_media_groups is not None:
            for g in interactive_media_groups:
                media = msg.textMessage.media.add()
                g.render(media)
        request = messaging_pb2.RequestSendMessage(
            peer=outpeer,
            deduplication_id=random.randint(0, 100000000),
            message=msg,
            is_only_for_user=uid
        )
        return self._send_message(request)

    def update_message(self, message, text, interactive_media_groups=None):
        """Update text message or interactive media (buttons, selects etc.).
        :param message: object received from any send method (send_message, send_file etc.)
        :param text: message text (not null)
        :param interactive_media_groups: groups of interactive media components (buttons etc.)
        :return: value of UpdateMessage response object
        """
        msg = messaging_pb2.MessageContent()
        msg.textMessage.text = text
        if interactive_media_groups is not None:
            for g in interactive_media_groups:
                media = msg.textMessage.media.add()
                g.render(media)

        return self._update(message, msg)

    def delete(self, message):
        """Delete text messages or interactive media (buttons, selects etc.).
        :param message: message object received from any send method (send_message, send_file etc.)
        """
        msg = messaging_pb2.MessageContent(
            deletedMessage=messaging_pb2.DeletedMessage(is_local=wrappers_pb2.BoolValue(value=False))
        )
        return self._update(message, msg)

    def get_messages_by_id(self, mids):
        result = self.internal.updates.GetReferencedEntitites(
            sequence_and_updates_pb2.RequestGetReferencedEntitites(
                mids=mids
            )
        )
        return result.messages

    def messages_read(self, peer, date):
        """Marking a message and all previous as read
        :param peer - chat peer
        :param date - date of message
        """
        request = messaging_pb2.RequestMessageRead(
            peer=peer,
            date=date
        )
        return self._read(request)

    def send_file(self, peer, file):
        """Send file to current peer.
        :param peer: receiver's peer
        :param file: path to file
        :return: value of SendMessage response object
        """
        if not peer:
            print('Peer can\'t be None!')
            return None

        location = self.internal.uploading.upload_file(file)
        outpeer = self.manager.get_outpeer(peer)
        msg = messaging_pb2.MessageContent()

        msg.documentMessage.CopyFrom(
            content.get_document_content(file, location)
        )

        request = messaging_pb2.RequestSendMessage(
            peer=outpeer,
            deduplication_id=random.randint(0, 100000000),
            message=msg
        )
        return self._send_message(request)

    def send_media(self, peer, medias):
        """Send media to current peer.
        :param peer: receiver's peer
        :param medias: medias (list)
        :return: value of SendMessage response object
        """
        if not peer:
            print('Peer can\'t be None!')
            return None
        outpeer = self.manager.get_outpeer(peer)
        text_message = messaging_pb2.TextMessage()
        for media in medias:
            text_message.media.append(media)
        msg = messaging_pb2.MessageContent(textMessage=text_message)
        request = messaging_pb2.RequestSendMessage(
            peer=outpeer,
            deduplication_id=random.randint(0, 100000000),
            message=msg
        )
        return self._send_message(request)

    def send_image(self, peer, file):
        """Send image as image (not as file) to current peer.
        :param peer: receiver's peer
        :param file: path to image file
        :return: value of SendMessage response object
        """
        if not peer:
            print('Peer can\'t be None!')
            return None

        if not is_image(file):
            raise IOError('File is not an image.')
        location = self.internal.uploading.upload_file(file)
        outpeer = self.manager.get_outpeer(peer)
        msg = messaging_pb2.MessageContent()

        msg.documentMessage.CopyFrom(
            content.get_image_content(file, location)
        )

        request = messaging_pb2.RequestSendMessage(
            peer=outpeer,
            deduplication_id=random.randint(0, 100000000),
            message=msg
        )

        return self._send_message(request)

    def send_audio(self, peer, file):
        """Send audio as audio (not as file) to current peer.
            :param peer: receiver's peer
            :param file: path to audio file
            :return: value of SendMessage response objectF
        """
        if not peer:
            print('Peer can\'t be None!')
            return None

        location = self.internal.uploading.upload_file(file)
        msg = messaging_pb2.MessageContent()

        msg.documentMessage.CopyFrom(
            content.get_audio_content(file, location)
        )

        request = messaging_pb2.RequestSendMessage(
                peer=peer,
                deduplication_id=random.randint(0, 100000000),
                message=msg
            )
        return self._send_message(request)

    def send_video(self, peer, file):
        """Send video as video (not as file) to current peer.
            :param peer: receiver's peer
            :param file: path to video file
            :return: value of SendMessage response object
        """
        if not peer:
            print('Peer can\'t be None!')
            return None

        if not is_video(file):
            raise IOError('File is not a video.')

        location = self.internal.uploading.upload_file(file)
        msg = messaging_pb2.MessageContent()

        msg.documentMessage.CopyFrom(
            content.get_video_content(file, location)
        )

        request = messaging_pb2.RequestSendMessage(
                peer=peer,
                deduplication_id=random.randint(0, 100000000),
                message=msg
            )
        return self._send_message(request)

    def reply(self, peer, mids, text=None, interactive_media_groups=None):
        """Reply message to current peer. Message can contain interactive media groups (buttons, selects etc.).
        :param mids: mids (array) of messages
        :param peer: receiver's peer
        :param text: message text (not null)
        :param interactive_media_groups: groups of interactive media components (buttons etc.)
        :return: value of SendMessage response object
        """

        if text is None:
            text = ''

        if not peer:
            print('Peer can\'t be None!')
            return None

        outpeer = self.manager.get_outpeer(peer)
        msg = messaging_pb2.MessageContent()
        msg.textMessage.text = text
        if interactive_media_groups is not None:
            for g in interactive_media_groups:
                media = msg.textMessage.media.add()
                g.render(media)
        request = messaging_pb2.RequestSendMessage(
            peer=outpeer,
            deduplication_id=random.randint(0, 100000000),
            message=msg,
            reply=messaging_pb2.ReferencedMessages(mids=mids)
        )
        return self._send_message(request)

    def forward(self, peer, mids, text=None, interactive_media_groups=None):
        """Forward message to current peer. Message can contain interactive media groups (buttons, selects etc.).
        :param peer: receiver's peer
        :param mids: mids (array) of messages
        :param text: message text (may be None)
        :param interactive_media_groups: groups of interactive media components (buttons etc.)
        :return: value of SendMessage response object
        """

        if text is None:
            text = ''

        if not peer:
            print('Peer can\'t be None!')
            return None

        outpeer, msg = self.get_outpeer_and_message(peer, text, interactive_media_groups)
        request = messaging_pb2.RequestSendMessage(
            peer=outpeer,
            deduplication_id=random.randint(0, 100000000),
            message=msg,
            forward=messaging_pb2.ReferencedMessages(mids=mids),
        )
        return self._send_message(request)

    def load_message_history(self, peer, date=0, direction=messaging_pb2.LISTLOADMODE_FORWARD, limit=2):
        if not peer:
            print('Peer can\'t be None!')
            return None

        outpeer = self.manager.get_outpeer(peer)
        request = messaging_pb2.RequestLoadHistory(
                peer=outpeer,
                date=date,
                load_mode=direction,
                limit=limit
            )
        return self._load_history(request)

    def on_message_async(self, callback, interactive_media_callback=None):
        updates_thread = threading.Thread(target=self.on_message, args=(callback, interactive_media_callback))
        updates_thread.start()

    def on_message(self, callback, interactive_media_callback=None, raw_callback=None):
        """Message receiving event handler.
        :param callback: function that will be called when message received
        :param interactive_media_callback: function that will be called when interactive media action is performed
        :param raw_callback: function to handle any other type of update
        :return: None
        """

        while True:
            try:
                for update in self.internal.updates.SeqUpdates(empty_pb2.Empty()):
                    up = sequence_and_updates_pb2.UpdateSeqUpdate()
                    up.ParseFromString(update.update.value)
                    if up.WhichOneof('update') == 'updateMessage':
                        self.internal.messaging.MessageReceived(messaging_pb2.RequestMessageReceived(
                            peer=self.manager.get_outpeer(up.updateMessage.peer),
                            date=up.updateMessage.date
                        ))
                        self.internal.messaging.MessageRead(messaging_pb2.RequestMessageRead(
                            peer=self.manager.get_outpeer(up.updateMessage.peer),
                            date=up.updateMessage.date
                        ))
                        self.internal.thread_pool_executor.submit(
                            callback(up.updateMessage)
                        )
                    elif up.WhichOneof('update') == 'updateInteractiveMediaEvent' and \
                            callable(interactive_media_callback):
                        self.internal.thread_pool_executor.submit(
                            interactive_media_callback(up.updateInteractiveMediaEvent)
                        )
                    else:
                        if callable(raw_callback):
                            self.internal.thread_pool_executor.submit(
                                raw_callback(up)
                            )
            except grpc.RpcError as e:
                if e.details() in ['Socket closed', 'GOAWAY received']:
                    continue

    def get_outpeer_and_message(self, peer, text, interactive_media_groups):
        outpeer = self.manager.get_outpeer(peer)
        msg = messaging_pb2.MessageContent()
        msg.textMessage.text = text
        if interactive_media_groups is not None:
            for g in interactive_media_groups:
                media = msg.textMessage.media.add()
                g.render(media)
        return outpeer, msg

    def _send_message(self, request):
        return self.internal.messaging.SendMessage(request)

    def _update_message(self, request):
        return self.internal.messaging.UpdateMessage(request)

    def _load_history(self, request):
        return self.internal.messaging.LoadHistory(request)

    def _read(self, request):
        return self.internal.messaging.MessageRead(request)

    def _update(self, message, new_message):
        if hasattr(message, "mid"):
            mid = message.mid
        elif hasattr(message, "message_id"):
            mid = message.message_id
        else:
            raise AttributeError("message has not attribute message_id or mid")

        # TODO: uncomment after 0.3.3 version
        # if message.edited_at.value:
        #     last_edited_at = message.edited_at.value
        # else:
        #     last_edited_at = int(time.time() * 1000)
        last_edited_at = int(time.time() * 1000)

        request = messaging_pb2.RequestUpdateMessage(
            mid=mid,
            updated_message=new_message,
            last_edited_at=last_edited_at
        )
        return self._update_message(request)



from dialog_api import peers_pb2


def private_peer(user_id):
    """Returns peer for user by user id

    :param user_id: user id
    :return: Peer object
    """
    return peers_pb2.Peer(type=peers_pb2.PEERTYPE_PRIVATE, id=user_id)


def group_peer(group_id):
    """Returns peer for group by group id

    :param group_id: group id
    :return: Peer object
    """
    return peers_pb2.Peer(type=peers_pb2.PEERTYPE_GROUP, id=group_id)


def peer_hasher(peer):
    """Returns a hash tuple of peer

    :param peer: Peer object
    :return: hash tuple
    """
    return peer.type, peer.id



import audioread
import mimetypes


def get_audio_duration(file):
    """Returns audio's duration

    :param file: path to audio
    :return: duration (int)
    """
    audio = audioread.audio_open(file)
    return int(audio.duration)


def is_audio(file):
    mime = mimetypes.guess_type(file)[0]
    return mime in ['audio/x-aiff', 'audio/basic', 'audio/mpeg', 'audio/x-pn-realaudio', 'audio/x-wav', 'audio/midi']



import copy
import unittest
from dialog_api import peers_pb2, messaging_pb2, definitions_pb2
from dialog_bot_sdk import interactive_media
from dialog_api.peers_pb2 import PEERTYPE_PRIVATE
from mock import patch

from tests.bot import bot
from dialog_bot_sdk.utils.get_media import get_webpage
from tests.fake_classes import FakeMessage


class TestMessaging(unittest.TestCase):

    test_file = "./files/test.png"
    test_image = test_file
    test_video = "./files/test.mov"
    test_audio = "./files/test.mp3"

    outpeer = peers_pb2.OutPeer(type=PEERTYPE_PRIVATE, id=0, access_hash=0)
    mid = definitions_pb2.UUIDValue(msb=0, lsb=0)
    msg_content = messaging_pb2.MessageContent()
    msg_content.textMessage.text = "Hello"
    group = [interactive_media.InteractiveMediaGroup(
        [
            interactive_media.InteractiveMedia(
                1,
                interactive_media.InteractiveMediaButton("Yes", "Да")
            ),
            interactive_media.InteractiveMedia(
                2,
                interactive_media.InteractiveMediaButton("No", "Нет")
            ),
        ]
    )]
    msg_content_with_group = copy.deepcopy(msg_content)
    group[0].render(msg_content_with_group.textMessage.media.add())
    doc_msg = messaging_pb2.DocumentMessage(
        file_id=0,
        access_hash=0,
        file_size=60,
        name=""
    )

    @patch('random.randint')
    @patch('dialog_bot_sdk.messaging.Messaging._send_message')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    def test_send_message(self, get_outpeer, send, rnd):
        self.assertIsNone(bot.messaging.send_message(None, "hello"))

        with self.assertRaises(AttributeError):
            self.assertRaises(bot.messaging.send_message(None, ""))

        get_outpeer.return_value = self.outpeer
        rnd.return_value = 1

        bot.messaging.send_message(self.outpeer, "Hello")

        self.assertTrue(isinstance(send.call_args.args[0], messaging_pb2.RequestSendMessage))

        bot.messaging.send_message(self.outpeer, "Hello", self.group)

        self.assertTrue(isinstance(send.call_args.args[0], messaging_pb2.RequestSendMessage))

    @patch('dialog_bot_sdk.messaging.Messaging._update_message')
    def test_update_message(self, update):
        message = FakeMessage(self.mid)

        bot.messaging.update_message(message, "Hello")

        self.assertTrue(isinstance(update.call_args.args[0], messaging_pb2.RequestUpdateMessage))

        bot.messaging.update_message(message, "Hello", self.group)

        args = update.call_args.args[0]
        self.assertTrue(isinstance(args, messaging_pb2.RequestUpdateMessage))

    @patch('random.randint')
    @patch('dialog_bot_sdk.messaging.Messaging._send_message')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    @patch('dialog_bot_sdk.uploading.Uploading.upload_file')
    def test_send_file(self, upload, get_outpeer, send, rnd):
        self.assertIsNone(bot.messaging.send_file(None, ""))
        upload.return_value = self.doc_msg
        get_outpeer.return_value = self.outpeer
        rnd.return_value = 1

        bot.messaging.send_file(self.outpeer, self.test_file)
        self.assertTrue(isinstance(send.call_args.args[0], messaging_pb2.RequestSendMessage))

    @patch('random.randint')
    @patch('dialog_bot_sdk.messaging.Messaging._send_message')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    def test_send_media(self, get_outpeer, send, rnd):
        self.assertIsNone(bot.messaging.send_media(None, ""))

        get_outpeer.return_value = self.outpeer
        rnd.return_value = 1

        media = [get_webpage("dlg.im")]

        bot.messaging.send_media(self.outpeer, media)
        self.assertTrue(isinstance(send.call_args.args[0], messaging_pb2.RequestSendMessage))

    @patch('random.randint')
    @patch('dialog_bot_sdk.messaging.Messaging._send_message')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    @patch('dialog_bot_sdk.uploading.Uploading.upload_file')
    def test_send_image(self, upload, get_outpeer, send, rnd):
        self.assertIsNone(bot.messaging.send_image(None, ""))
        with self.assertRaises(IOError):
            self.assertRaises(bot.messaging.send_image("peer", ""))

        upload.return_value = self.doc_msg
        get_outpeer.return_value = self.outpeer
        rnd.return_value = 1

        with self.assertRaises(IOError):
            bot.messaging.send_image(self.outpeer, self.test_audio)

        bot.messaging.send_image(self.outpeer, self.test_image)
        self.assertTrue(isinstance(send.call_args.args[0], messaging_pb2.RequestSendMessage))

    @patch('random.randint')
    @patch('dialog_bot_sdk.messaging.Messaging._send_message')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    @patch('dialog_bot_sdk.uploading.Uploading.upload_file')
    def test_send_audio(self, upload, get_outpeer, send, rnd):
        self.assertIsNone(bot.messaging.send_audio(None, ""))
        with self.assertRaises(IOError):
            self.assertRaises(bot.messaging.send_audio("peer", ""))

        upload.return_value = self.doc_msg
        get_outpeer.return_value = self.outpeer
        rnd.return_value = 1

        with self.assertRaises(IOError):
            bot.messaging.send_audio(self.outpeer, self.test_image)

        bot.messaging.send_audio(self.outpeer, self.test_audio)
        self.assertTrue(isinstance(send.call_args.args[0], messaging_pb2.RequestSendMessage))

    @patch('random.randint')
    @patch('dialog_bot_sdk.messaging.Messaging._send_message')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    @patch('dialog_bot_sdk.uploading.Uploading.upload_file')
    def test_send_video(self, upload, get_outpeer, send, rnd):
        self.assertIsNone(bot.messaging.send_video(None, ""))
        with self.assertRaises(IOError):
            self.assertRaises(bot.messaging.send_video("peer", ""))
        upload.return_value = self.doc_msg
        get_outpeer.return_value = self.outpeer
        rnd.return_value = 1

        with self.assertRaises(IOError):
            bot.messaging.send_video(self.outpeer, self.test_image)

        bot.messaging.send_video(self.outpeer, self.test_video)
        self.assertTrue(isinstance(send.call_args.args[0], messaging_pb2.RequestSendMessage))

    @patch('random.randint')
    @patch('dialog_bot_sdk.messaging.Messaging._send_message')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    def test_reply(self, get_outpeer, send, rnd):
        self.assertIsNone(bot.messaging.reply(None, ""))

        get_outpeer.return_value = self.outpeer
        rnd.return_value = 1

        bot.messaging.reply(self.outpeer, [self.mid])

        self.assertTrue(isinstance(send.call_args.args[0], messaging_pb2.RequestSendMessage))

        bot.messaging.reply(self.outpeer, [self.mid], "hello", self.group)

        self.assertTrue(isinstance(send.call_args.args[0], messaging_pb2.RequestSendMessage))

    @patch('dialog_bot_sdk.messaging.Messaging._load_history')
    def test_load_message_history(self, load):
        self.assertIsNone(bot.messaging.load_message_history(None))
        bot.messaging.load_message_history(self.outpeer)
        self.assertTrue(isinstance(load.call_args.args[0], messaging_pb2.RequestLoadHistory))
        bot.messaging.load_message_history(self.outpeer, 1)
        self.assertTrue(isinstance(load.call_args.args[0], messaging_pb2.RequestLoadHistory))
        bot.messaging.load_message_history(self.outpeer, 1, messaging_pb2.LISTLOADMODE_BACKWARD)
        self.assertTrue(isinstance(load.call_args.args[0], messaging_pb2.RequestLoadHistory))
        bot.messaging.load_message_history(self.outpeer, 1, messaging_pb2.LISTLOADMODE_BACKWARD, 1)
        self.assertTrue(isinstance(load.call_args.args[0], messaging_pb2.RequestLoadHistory))

    @patch('dialog_bot_sdk.messaging.Messaging._delete')
    def test_delete(self, delete):
        bot.messaging.delete([self.mid])
        self.assertTrue(isinstance(delete.call_args.args[0], messaging_pb2.RequestDeleteMessageObsolete))

    @patch('dialog_bot_sdk.messaging.Messaging._read')
    def test_read(self, read):
        bot.messaging.messages_read(self.outpeer, 0)
        self.assertTrue(isinstance(read.call_args.args[0], messaging_pb2.RequestMessageRead))


if __name__ == '__main__':
    unittest.main()


def read_file_in_chunks(file, chunk_size=1024*1024):
    """File chunks generator

    :param file: path to file
    :param chunk_size: size of a chunk
    :return: generator object that iterates over file chunks
    """

    file_object = open(file, 'rb')
    while True:
        data = file_object.read(chunk_size)
        if not data:
            file_object.close()
            break
        yield data



import unittest

from dialog_api import messaging_pb2, peers_pb2, contacts_pb2, users_pb2, search_pb2
from mock import patch
from tests.bot import bot
from tests.fake_classes import FakeDialog, FakeUsers, FakeUser, FakeFullProfile, FakeFullUser


class TestUsers(unittest.TestCase):
    outpeer = peers_pb2.OutPeer(
        type=peers_pb2.PEERTYPE_PRIVATE,
        id=0,
        access_hash=0
    )

    def test_get_user_peer_by_id(self):
        self.assertTrue(isinstance(bot.users.get_user_peer_by_id(0), peers_pb2.Peer))

    @patch('dialog_bot_sdk.users.Users._load_dialogs')
    def test_get_user_outpeer_by_id(self, load):
        self.assertIsNone(bot.users.get_user_outpeer_by_id(0), peers_pb2.Peer)
        self.assertTrue(isinstance(load.call_args.args[0], messaging_pb2.RequestLoadDialogs))
        load.return_value = FakeDialog()
        self.assertTrue(isinstance(bot.users.get_user_outpeer_by_id(0), peers_pb2.OutPeer))

    @patch('dialog_bot_sdk.users.Users._search_contacts')
    def test_find_user_outpeer_by_nick(self, search):
        search.return_value = FakeUsers("bot")
        self.assertTrue(isinstance(bot.users.find_user_outpeer_by_nick("bot"), peers_pb2.OutPeer))
        self.assertTrue(isinstance(search.call_args.args[0], contacts_pb2.RequestSearchContacts))
        self.assertIsNone(bot.users.find_user_outpeer_by_nick("ne_bot"))

    @patch('dialog_bot_sdk.users.Users._search_contacts')
    def test_get_user_by_nick(self, search):
        search.return_value = FakeUsers("bot")
        self.assertTrue(isinstance(bot.users.get_user_by_nick("bot"), FakeUser))
        self.assertTrue(isinstance(search.call_args.args[0], contacts_pb2.RequestSearchContacts))
        self.assertIsNone(bot.users.get_user_by_nick("ne_bot"))

    @patch('dialog_bot_sdk.users.Users._load_full_users')
    @patch('dialog_bot_sdk.users.Users.find_user_outpeer_by_nick')
    def test_get_user_full_profile_by_nick(self, find, load):
        find.return_value = self.outpeer
        load.return_value = FakeFullProfile()
        self.assertIsNone(bot.users.get_user_full_profile_by_nick("bot"))
        self.assertTrue(isinstance(load.call_args.args[0], users_pb2.RequestLoadFullUsers))
        load.return_value = FakeFullProfile([FakeFullUser("custom_profile")])
        self.assertTrue(isinstance(bot.users.get_user_full_profile_by_nick("bot"), FakeFullUser))

    @patch('dialog_bot_sdk.users.Users.get_user_full_profile_by_nick')
    def test_get_user_custom_profile_by_nick(self, profile):
        profile.return_value = FakeFullUser()
        self.assertIsNone(bot.users.get_user_custom_profile_by_nick("bot"))
        profile.return_value = FakeFullUser("custom_profile")
        self.assertEqual(bot.users.get_user_custom_profile_by_nick("bot"), "custom_profile")

    @patch('dialog_bot_sdk.users.Users._load_full_users')
    @patch('dialog_bot_sdk.entity_manager.EntityManager.get_outpeer')
    def test_get_user_custom_profile_by_peer(self, get_outpeer, load):
        get_outpeer.return_value = self.outpeer
        load.return_value = FakeFullProfile()
        self.assertIsNone(bot.users.get_user_custom_profile_by_peer("bot"))
        self.assertTrue(isinstance(load.call_args.args[0], users_pb2.RequestLoadFullUsers))
        load.return_value = FakeFullProfile([FakeFullUser()])
        self.assertIsNone(bot.users.get_user_custom_profile_by_peer("bot"))
        load.return_value = FakeFullProfile([FakeFullUser("custom_profile")])
        self.assertEqual(bot.users.get_user_custom_profile_by_peer("bot"), "custom_profile")

    @patch('dialog_bot_sdk.users.Users._peer_search')
    def test_search_users_by_nick_substring(self, search):
        search.return_value = FakeUsers("bot")
        self.assertTrue(isinstance(bot.users.search_users_by_nick_substring("bot")[0], FakeUser))
        self.assertTrue(isinstance(search.call_args.args[0], search_pb2.RequestPeerSearch))

    def test_get_user_outpeer_by_outpeer(self):
        self.assertTrue(isinstance(bot.users.get_user_outpeer_by_outpeer(self.outpeer), peers_pb2.UserOutPeer))


if __name__ == '__main__':
    unittest.main()



import unittest

from dialog_api import sequence_and_updates_pb2
from mock import patch

from tests.bot import bot
from tests.fake_classes import FakeState


class TestUpdates(unittest.TestCase):
    @patch('dialog_bot_sdk.updates.Updates._get_difference')
    def test_get_difference(self, diff):
        bot.updates.get_difference(1337)
        self.assertTrue(isinstance(diff.call_args.args[0], sequence_and_updates_pb2.RequestGetDifference))

    @patch('dialog_bot_sdk.updates.Updates._get_state')
    def test_get_state(self, state):
        state.return_value = FakeState()
        self.assertEqual(bot.updates.get_state(), 1)
        self.assertTrue(isinstance(state.call_args.args[0], sequence_and_updates_pb2.RequestGetState))


if __name__ == '__main__':
    unittest.main()



import unittest

from dialog_api import media_and_files_pb2
from mock import patch

from tests.bot import bot
from tests.fake_classes import FakeUrl, FakePut
from dialog_bot_sdk import uploading


class TestUploading(unittest.TestCase):
    file = "./files/test.png"

    @patch('requests.put')
    @patch('dialog_bot_sdk.uploading.Uploading._get_file_upload_part_url')
    def test_upload_file_chunk(self, get_url, put):
        get_url.return_value = FakeUrl()
        put.return_value = FakePut(400)
        self.assertIsNone(bot.internal.uploading.upload_file_chunk(0, "chunk", b'key'))
        put.return_value = FakePut(200)
        self.assertTrue(isinstance(bot.internal.uploading.upload_file_chunk(0, "chunk", b'key'), FakePut))
        self.assertTrue(isinstance(get_url.call_args.args[0], media_and_files_pb2.RequestGetFileUploadPartUrl))
        self.assertEqual(put.call_args.args[0], "url")
        self.assertEqual(put.call_args_list[0].kwargs['data'], "chunk")
        self.assertEqual(put.call_args_list[0].kwargs['headers'], {'Content-Type': 'application/octet-stream'})
        upload = uploading.Uploading(bot.internal, b'cert', b'private_key', "access_dir")
        self.assertTrue(isinstance(upload.upload_file_chunk(0, "chunk", b'key'), FakePut))

    @patch('dialog_bot_sdk.uploading.Uploading.upload_file_chunk')
    @patch('dialog_bot_sdk.uploading.Uploading._commit_file_upload')
    @patch('dialog_bot_sdk.uploading.Uploading._get_file_upload_url')
    def test_upload_file(self, get_url, commit, chunk):
        upload = uploading.Uploading(bot.internal, b'cert', b'private_key', "access_dir")
        get_url.return_value = FakePut(upload_key=b'upload_key')
        commit.return_value = FakePut(uploaded_file_location="uploaded_file_location")
        self.assertEqual(upload.upload_file(self.file), "uploaded_file_location")
        self.assertTrue(isinstance(get_url.call_args.args[0], media_and_files_pb2.RequestGetFileUploadUrl))
        self.assertTrue(isinstance(commit.call_args.args[0], media_and_files_pb2.RequestCommitFileUpload))
        self.assertEqual(chunk.call_args.args[2], b'upload_key')
        self.assertEqual(chunk.call_args.args[3], "access_dir")
        chunk.return_value = None
        self.assertIsNone(upload.upload_file(self.file))


if __name__ == '__main__':
    unittest.main()



import unittest

from dialog_api import groups_pb2, peers_pb2, search_pb2
from mock import patch

from tests.bot import bot
from tests.fake_classes import FakeSearch


class TestGroups(unittest.TestCase):
    @patch('dialog_bot_sdk.groups.Groups._create_group')
    def test_create_group(self, create):
        bot.groups.create_group("title", "username")
        self.assertTrue(isinstance(create.call_args.args[0], groups_pb2.RequestCreateGroup))
        bot.groups.create_group("title", "username", [peers_pb2.UserOutPeer(
            uid=0,
            access_hash=0
        )])
        self.assertTrue(isinstance(create.call_args.args[0], groups_pb2.RequestCreateGroup))

    @patch('dialog_bot_sdk.groups.Groups._peer_search')
    def test_find_group_by_shortname(self, search):
        search.return_value = FakeSearch()
        self.assertEqual(bot.groups.find_group_by_shortname("shortname"), ["group"])
        self.assertTrue(isinstance(search.call_args.args[0], search_pb2.RequestPeerSearch))


if __name__ == '__main__':
    unittest.main()



import unittest

import grpc
from mock import patch

from dialog_bot_sdk.bot import DialogBot
from dialog_bot_sdk.internal.bot import InternalBot


class TestBot(unittest.TestCase):
    @patch('dialog_bot_sdk.entity_manager.EntityManager.bootstrap')
    @patch('dialog_bot_sdk.internal.bot.InternalBot.get_session_token')
    @patch('dialog_bot_sdk.internal.bot.InternalBot.anonymous_authorize')
    def test_get_insecure_bot(self, anon, token, boot):
        token.return_value = "token"
        self.assertTrue(isinstance(DialogBot.get_insecure_bot("endpoint", None), DialogBot))
        self.assertTrue(anon.called)
        self.assertTrue(token.called)
        self.assertTrue(boot.called)

    @patch('dialog_bot_sdk.entity_manager.EntityManager.bootstrap')
    @patch('dialog_bot_sdk.internal.bot.InternalBot.get_session_token')
    @patch('dialog_bot_sdk.internal.bot.InternalBot.authorize')
    def test_get_secure_bot(self, auth, token, boot):
        token.return_value = "token"
        self.assertTrue(isinstance(DialogBot.get_secure_bot("endpoint", grpc.ssl_channel_credentials(), "token"),
                                   DialogBot))
        self.assertEqual(auth.call_args.args[0], "token")
        self.assertTrue(token.called)


if __name__ == '__main__':
    unittest.main()



import unittest

from tests.test_users import TestUsers
from tests.test_messaning import TestMessaging
from tests.test_peers import TestPeers
from tests.test_groups import TestGroups
from tests.test_updates import TestUpdates
from tests.test_manager import TestManager
from tests.test_interactive import TestInteractive
from tests.test_uploading import TestUploading
from tests.test_bot import TestBot
from tests.test_get_media import TestGetMedia

if __name__ == '__main__':
    unittest.main()
